@@ -1,5 +1,5 @@
-use euclid::Size2D;
-use wgpu::{util::DeviceExt, FilterMode, ShaderFlags, TextureFormat, TextureUsage};
+use figures::{Pixels, Size};
+use wgpu::{util::DeviceExt, FilterMode, MultisampleState, TextureFormat, TextureUsages};
 
 use crate::{
     binding::{Bind, Binding, BindingGroup, BindingGroupLayout},
@@ -7,7 +7,6 @@ use crate::{
     pipeline::{Blending, Pipeline, PipelineLayout, Set},
     sampler::Sampler,
     shader::Shader,
-    swapchain::SwapChain,
     texture::Texture,
     transform::ScreenSpace,
     vertex::VertexLayout,
@@ -18,6 +17,7 @@ pub struct Device {
     pub wgpu: wgpu::Device,
     pub queue: wgpu::Queue,
     pub surface: Option<wgpu::Surface>,
+    size: Size<u32, Pixels>,
 }
 
 impl Device {
@@ -40,6 +40,7 @@ impl Device {
             wgpu: device,
             queue,
             surface: Some(surface),
+            size: Size::default(),
         })
     }
 
@@ -59,13 +60,18 @@ impl Device {
             wgpu: device,
             queue,
             surface: None,
+            size: Size::default(),
         })
     }
 
-    pub fn device(&self) -> &wgpu::Device {
+    pub const fn device(&self) -> &wgpu::Device {
         &self.wgpu
     }
 
+    pub const fn size(&self) -> Size<u32, Pixels> {
+        self.size
+    }
+
     pub fn device_mut(&mut self) -> &mut wgpu::Device {
         &mut self.wgpu
     }
@@ -75,19 +81,24 @@ impl Device {
             .create_command_encoder(&wgpu::CommandEncoderDescriptor { label: None })
     }
 
-    pub fn create_swap_chain<PM: Into<wgpu::PresentMode>>(
-        &self,
-        size: Size2D<u32, ScreenSpace>,
+    pub fn configure<PM: Into<wgpu::PresentMode>>(
+        &mut self,
+        size: Size<u32, ScreenSpace>,
         mode: PM,
         format: TextureFormat,
-    ) -> wgpu::SwapChain {
-        let desc = SwapChain::descriptor(size, mode, format);
-        self.wgpu.create_swap_chain(
-            self.surface
-                .as_ref()
-                .expect("create_swap_chain only works when initalized with a wgpu::Surface"),
-            &desc,
-        )
+    ) {
+        let desc = wgpu::SurfaceConfiguration {
+            usage: wgpu::TextureUsages::RENDER_ATTACHMENT,
+            format,
+            present_mode: mode.into(),
+            width: size.width,
+            height: size.height,
+        };
+        self.surface
+            .as_ref()
+            .expect("create_swap_chain only works when initalized with a wgpu::Surface")
+            .configure(&self.wgpu, &desc);
+        self.size = size;
     }
 
     pub fn create_pipeline_layout(&self, ss: &[Set]) -> PipelineLayout {
@@ -104,7 +115,6 @@ impl Device {
                 .wgpu
                 .create_shader_module(&wgpu::ShaderModuleDescriptor {
                     source: wgpu::util::make_spirv(source),
-                    flags: ShaderFlags::default(),
                     label: None, // TODO labels would be nice
                 }),
         }
@@ -116,7 +126,6 @@ impl Device {
                 .wgpu
                 .create_shader_module(&wgpu::ShaderModuleDescriptor {
                     source: wgpu::ShaderSource::Wgsl(source.into()),
-                    flags: ShaderFlags::default(),
                     label: None, // TODO labels would be nice
                 }),
         }
@@ -129,9 +138,10 @@ impl Device {
 
     pub fn create_texture(
         &self,
-        size: Size2D<u32, ScreenSpace>,
+        size: Size<u32, ScreenSpace>,
         format: TextureFormat,
-        usage: TextureUsage,
+        usage: TextureUsages,
+        sample_count: u32,
     ) -> Texture {
         let texture_extent = wgpu::Extent3d {
             width: size.width,
@@ -141,7 +151,7 @@ impl Device {
         let texture = self.wgpu.create_texture(&wgpu::TextureDescriptor {
             size: texture_extent,
             mip_level_count: 1,
-            sample_count: 1,
+            sample_count,
             dimension: wgpu::TextureDimension::D2,
             format,
             usage,
@@ -160,8 +170,9 @@ impl Device {
 
     pub fn create_framebuffer(
         &self,
-        size: Size2D<u32, ScreenSpace>,
+        size: Size<u32, ScreenSpace>,
         format: TextureFormat,
+        sample_count: u32,
     ) -> Framebuffer {
         let extent = wgpu::Extent3d {
             width: size.width,
@@ -171,13 +182,13 @@ impl Device {
         let texture = self.wgpu.create_texture(&wgpu::TextureDescriptor {
             size: extent,
             mip_level_count: 1,
-            sample_count: 1,
+            sample_count,
             dimension: wgpu::TextureDimension::D2,
             format,
-            usage: TextureUsage::SAMPLED
-                | TextureUsage::COPY_DST
-                | TextureUsage::COPY_SRC
-                | TextureUsage::RENDER_ATTACHMENT,
+            usage: TextureUsages::TEXTURE_BINDING
+                | TextureUsages::COPY_DST
+                | TextureUsages::COPY_SRC
+                | TextureUsages::RENDER_ATTACHMENT,
             label: None,
         });
         let view = texture.create_view(&wgpu::TextureViewDescriptor::default());
@@ -190,11 +201,11 @@ impl Device {
                 format,
                 size,
             },
-            depth: self.create_zbuffer(size),
+            depth: self.create_zbuffer(size, sample_count),
         }
     }
 
-    pub fn create_zbuffer(&self, size: Size2D<u32, ScreenSpace>) -> DepthBuffer {
+    pub fn create_zbuffer(&self, size: Size<u32, ScreenSpace>, sample_count: u32) -> DepthBuffer {
         let format = DepthBuffer::FORMAT;
         let extent = wgpu::Extent3d {
             width: size.width,
@@ -205,10 +216,10 @@ impl Device {
             size: extent,
             label: None,
             mip_level_count: 1,
-            sample_count: 1,
+            sample_count,
             dimension: wgpu::TextureDimension::D2,
             format,
-            usage: TextureUsage::COPY_DST | TextureUsage::RENDER_ATTACHMENT,
+            usage: TextureUsages::COPY_DST | TextureUsages::RENDER_ATTACHMENT,
         });
         let view = wgpu.create_view(&wgpu::TextureViewDescriptor::default());
 
@@ -255,7 +266,7 @@ impl Device {
         T: 'static + Copy,
     {
         VertexBuffer {
-            wgpu: self.create_buffer_from_slice(vertices, wgpu::BufferUsage::VERTEX),
+            wgpu: self.create_buffer_from_slice(vertices, wgpu::BufferUsages::VERTEX),
             size: (vertices.len() * std::mem::size_of::<T>()) as u32,
         }
     }
@@ -272,13 +283,13 @@ impl Device {
                 .create_buffer_init(&wgpu::util::BufferInitDescriptor {
                     label: Some("Uniform Buffer"),
                     contents: bytemuck::cast_slice(buf),
-                    usage: wgpu::BufferUsage::UNIFORM | wgpu::BufferUsage::COPY_DST,
+                    usage: wgpu::BufferUsages::UNIFORM | wgpu::BufferUsages::COPY_DST,
                 }),
         }
     }
 
     pub fn create_index(&self, indices: &[u16]) -> IndexBuffer {
-        let index_buf = self.create_buffer_from_slice(indices, wgpu::BufferUsage::INDEX);
+        let index_buf = self.create_buffer_from_slice(indices, wgpu::BufferUsages::INDEX);
         IndexBuffer {
             wgpu: index_buf,
             elements: indices.len() as u32,
@@ -296,7 +307,7 @@ impl Device {
                 mipmap_filter: wgpu::FilterMode::Nearest,
                 lod_min_clamp: -100.0,
                 lod_max_clamp: 100.0,
-                compare: Some(wgpu::CompareFunction::Always),
+                compare: None,
                 anisotropy_clamp: None,
                 label: None,
                 border_color: None,
@@ -327,7 +338,7 @@ impl Device {
     pub fn create_buffer_from_slice<T: bytemuck::Pod>(
         &self,
         slice: &[T],
-        usage: wgpu::BufferUsage,
+        usage: wgpu::BufferUsages,
     ) -> wgpu::Buffer {
         self.wgpu
             .create_buffer_init(&wgpu::util::BufferInitDescriptor {
@@ -350,6 +361,8 @@ impl Device {
         self.queue.submit(cmds);
     }
 
+    // TODO clippy::too_many_arguments
+    #[allow(clippy::too_many_arguments)]
     pub fn create_pipeline(
         &self,
         pipeline_layout: PipelineLayout,
@@ -358,6 +371,7 @@ impl Device {
         vs: &Shader,
         fs: &Shader,
         swapchain_format: TextureFormat,
+        multisample: MultisampleState,
     ) -> Pipeline {
         let vertex_attrs = vertex_layout.to_wgpu();
 
@@ -373,7 +387,7 @@ impl Device {
                 push_constant_ranges: &[],
             });
 
-        let (src_factor, dst_factor, operation) = blending.to_wgpu();
+        let (src_factor, dst_factor, operation) = blending.as_wgpu();
 
         let wgpu = self
             .wgpu
@@ -410,7 +424,7 @@ impl Device {
                         clamp: 0.,
                     },
                 }),
-                multisample: wgpu::MultisampleState::default(),
+                multisample,
                 fragment: Some(wgpu::FragmentState {
                     module: &fs.wgpu,
                     entry_point: "main",
@@ -429,7 +443,7 @@ impl Device {
                             },
                         }),
 
-                        write_mask: wgpu::ColorWrite::ALL,
+                        write_mask: wgpu::ColorWrites::ALL,
                     }],
                 }),
             });
