@@ -12,18 +12,17 @@ pub mod renderable;
 pub mod renderer;
 pub mod sampler;
 pub mod shader;
-pub mod swapchain;
 pub mod texture;
 pub mod transform;
 pub mod vertex;
 
-pub use euclid;
+pub use figures;
 pub use wgpu;
 
 pub mod prelude {
     pub use super::{
-        binding::*, buffers::*, canvas::*, color::*, device::*, error::*, euclid, frame::*,
-        pipeline::*, renderable::*, renderer::*, sampler::*, shader::*, swapchain::*, texture::*,
-        transform::*, vertex::*, wgpu,
+        binding::*, buffers::*, canvas::*, color::*, device::*, error::*, frame::*, pipeline::*,
+        renderable::*, renderer::*, sampler::*, shader::*, texture::*, transform::*, vertex::*,
+        wgpu,
     };
 }
